@@ -1,4 +1,4 @@
-// Copyright 2007-2022 The Mumble Developers. All rights reserved.
+// Copyright 2007-2023 The Mumble Developers. All rights reserved.
 // Use of this source code is governed by a BSD-style license
 // that can be found in the LICENSE file at the root of the
 // Mumble source tree or at <https://www.mumble.info/LICENSE>.
@@ -55,10 +55,10 @@ struct Shortcut {
 };
 
 struct ChannelTarget {
-	int channelID = Channel::ROOT_ID;
+	unsigned int channelID = Channel::ROOT_ID;
 
 	ChannelTarget() = default;
-	ChannelTarget(int id) : channelID(id) {}
+	ChannelTarget(unsigned int id) : channelID(id) {}
 
 	friend bool operator==(const ChannelTarget &lhs, const ChannelTarget &rhs);
 	friend bool operator<(const ChannelTarget &lhs, const ChannelTarget &rhs);
@@ -220,11 +220,13 @@ struct Settings {
 	quint64 uiDoublePush     = 0;
 	quint64 pttHold          = 0;
 
-	bool bTxAudioCue        = false;
+	bool audioCueEnabledPTT = true;
+	bool audioCueEnabledVAD = false;
 	QString qsTxAudioCueOn  = cqsDefaultPushClickOn;
 	QString qsTxAudioCueOff = cqsDefaultPushClickOn;
 
 	bool bTxMuteCue     = true;
+	bool muteCueShown   = false;
 	QString qsTxMuteCue = cqsDefaultMuteCue;
 
 	bool bTransmitPosition         = false;
@@ -335,7 +337,7 @@ struct Settings {
 	bool bPositionalHeadphone     = false;
 	float fAudioMinDistance       = 1.0f;
 	float fAudioMaxDistance       = 15.0f;
-	float fAudioMaxDistVolume     = 0.25f;
+	float fAudioMaxDistVolume     = 0.0f;
 	float fAudioBloom             = 0.5f;
 	/// Contains the settings for each individual plugin. The key in this map is the Hex-represented SHA-1
 	/// hash of the plugin's UTF-8 encoded absolute file-path on the hard-drive.
@@ -368,6 +370,7 @@ struct Settings {
 
 	QPoint qpTalkingUI_Position              = UNSPECIFIED_POSITION;
 	bool bShowTalkingUI                      = false;
+	bool talkingUI_UsersAlwaysVisible        = false;
 	bool bTalkingUI_LocalUserStaysVisible    = false;
 	bool bTalkingUI_AbbreviateChannelNames   = true;
 	bool bTalkingUI_AbbreviateCurrentChannel = false;
